/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <sys/resource.h>
#include <string.h>

#include <ostore/ostore.hh>
#include <ostore/coll.hh>
#include <ostore/coll/plist.hh>
#include <ostore/coll/iplist.hh>
#include <ostore/relat.hh>
#include "OO7.h"
#include "GenParams.h"
#include "VarParams.h"

extern os_database* oo7db;

extern int	debugMode;
extern char*	types[NumTypes];


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart Constructor
//
////////////////////////////////////////////////////////////////////////////

AtomicPart::AtomicPart(int ptId) : id(ptId)
{
    if (debugMode) {
        printf("BEGIN AtomicPart::AtomicPart(ptId = %d)\n", ptId);
    }
    
    // initialize internal state of new part (most of it randomly)
    int typeNo = (int) (random() % NumTypes);
    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinAtomicDate +
    		(int) (random() % (MaxAtomicDate-MinAtomicDate+1));
    x = (int) (random() % XYRange);
    y = (int) (random() % XYRange);

    docId = (int) random() % TotalCompParts + 1;

    if (debugMode) {
        printf("MIDDLE AtomicPart::AtomicPart(ptId = %d)\n", ptId);
        printf("\tbuildDate = %d\n", buildDate);
        printf("\tx, y = %d, %d\n", x, y);
    }

    // add part to AtomicPart extent
    (*AtomicPart::extent) |= this;

    if (debugMode) {
        printf("END AtomicPart::AtomicPart(ptId = %d)\n", ptId);
    }
}


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart swapXY Method for use in update traversals
//
////////////////////////////////////////////////////////////////////////////

void AtomicPart::swapXY()
{
    if (debugMode) {
        printf("                    AtomicPart::swapXY(ptId = %d, x = %d, y = %d)\n", id,x,y);
    }

    // exchange X and Y values

    int tmp = x;
    x = y;
    y = tmp;

    if (debugMode) {
        printf("                    [did swap, so x = %d, y = %d]\n", x, y);
    }
}


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart toggleDate Method for use in update traversals
//
////////////////////////////////////////////////////////////////////////////

void AtomicPart::toggleDate()
{
    if (debugMode) {
        printf("                    AtomicPart::toggleDate(ptId = %d, buildDate = %d)\n",
	       id, buildDate);
    }

    // increment build date if odd, decrement it if even

    if (buildDate % 2) {
	// odd case
	buildDate++;
    } else {
	// even case
	buildDate--;
    }

    if (debugMode) {
        printf("                    [did toggle, so buildDate = %d]\n", buildDate);
    }
}

////////////////////////////////////////////////////////////////////////////
//
// AtomicPart DoNothing Method 
//
////////////////////////////////////////////////////////////////////////////

int RealWork; // set to one to make DoNothing do work.
int WorkAmount; // amount of work to do

void AtomicPart::DoNothing()
{
    struct timeval tv;
    struct timezone tz;

    int i;
    if (id < 0)
	   printf("DoNothing - negative id\n");

    if (debugMode) {
        printf("==> DoNothing(x = %d, y = %d, type = %s)\n", x, y, type);
    }

    if (RealWork) {
            for (i = 0; i < WorkAmount; i++) {
               gettimeofday(&tv, &tz);
            }
    }
}


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart Destructor
//
////////////////////////////////////////////////////////////////////////////

AtomicPart::~AtomicPart()
{
    if (debugMode) {
        printf("Entering AtomicPart::~AtomicPart(id = %d)\n", id);
    }

    // delete all affected connections
if (debugMode) {
    printf("    In ~AtomicPart, about to delete 'to' connections\n");
}
    Connection* c;
    while (!to.empty()) {
	c = to.pick();
        delete c;
    }
if (debugMode) {
    printf("    In ~AtomicPart, about to delete 'from' connections\n");
}
    while (!from.empty()) {
	c = from.pick();
        delete c;
    }

    // remove part from AtomicPart extent
if (debugMode) {
    printf("    In ~AtomicPart, about to remove part from AtomicPart::extent\n");
}
    (*AtomicPart::extent) -= this;

    if (debugMode) {
        printf("Leaving AtomicPart::~AtomicPart(id = %d)\n", id);
    }
}

