/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


#include <libc.h>
#include <string.h>
#include <stdio.h>

#include <ostore/ostore.hh>
#include <ostore/coll.hh>
#include <ostore/relat.hh>
#include "OO7.h"
#include "GenParams.h"
#include "BenchParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #3 - 10% selection on AtomicParts via build date (the
// most recent 10% of AtomicParts.)   The actual percentage
// is determined by the constant Query3Percent.  This may
// be varied as we gain experience with the benchmark - the
// goal is to check out if the "optimizer" is smart enough
// to properly choose between B+ tree and scan. 
//
/////////////////////////////////////////////////////////////////

extern os_database* oo7db;

extern int	debugMode;

//////////////////////////////////////////////////////////////////
//
// Routine to do Query #3
//
//////////////////////////////////////////////////////////////////


int query3()
{
    // choose a range of dates with the appropriate selectivity

    int dateRange = (Query3Percent * (MaxAtomicDate - MinAtomicDate)) / 100;
    int lowerDate = MaxAtomicDate - dateRange;

    if (debugMode) {
        printf("    Running Query3 with dateRange = %d, lowerDate = %d\n",
		    dateRange, lowerDate);
    }

    // first find the desired parts

    os_Set<AtomicPart*>& rslt =
                            (*AtomicPart::extent)[: buildDate >= lowerDate :];

    // now process the relevant parts

    int count = 0;
    foreach (AtomicPart* atom, rslt) {
        if (debugMode) {
            printf("    In Query3, partId = %d, buildDate = %d\n",
	                atom->id, atom->buildDate);
        }
        atom->DoNothing();
	count++;
    }
    return count;
}

